@@ -42,7 +42,6 @@ twostagelvm <- function(object, model2,
     return(res)
 }
 
-
 uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
     if (!is.function(nlobj)) {
         predict.fun <- lapply(nlobj, function(x) x[["pred"]])
@@ -80,7 +79,6 @@ uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
     return(cbind(predict.fun(P$mean, P$var, model.frame(model1))))
 }
 
-
 ##' Two-stage estimator
 ##'
 ##' Generic function.
@@ -188,7 +186,7 @@ uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
 ##'  mb4 <- nonlinear(mb,type="spline",knots=c(-3,-2,-1,0,1,2,3),y~u)
 ##'  ff <- lapply(list(mb1,mb2,mb3,mb4),
 ##'	      function(m) function(data,...) twostage(ma,m,data=data,st.derr=FALSE))
-##'  a <- cv(ff,data=d,rep=1,mc.cores=1)
+##'  a <- cv(ff,data=d,rep=1)
 ##'  a
 ##'}
 twostage.lvmfit <- function(object, model2, data=NULL,
@@ -220,7 +218,7 @@ twostage.lvmfit <- function(object, model2, data=NULL,
         if (is.null(id2)) id2 <- seq(nrow(model.frame(model2)))
         model1 <- object
         if (!inherits(object,"estimate")) {
-            model1 <- estimate(NULL,coef=p1,id=id1,iid=iid(object))
+            model1 <- estimate(NULL, coef=p1, id=id1, IC=IC(object))
         }
 
         e2 <- estimate(model2, id=id2)
@@ -278,7 +276,6 @@ estimate.twostage.lvm <- function(x,data,...) {
 ##' @export
 twostage.twostage.lvm <- function(object,...) estimate.twostage.lvm(object,...)
 
-
 ##' @export
 twostage.lvm <- function(object,model2,data=NULL, ...) {
     if (is.null(data)) {
@@ -304,7 +301,6 @@ print.twostage.lvm <- function(x,...) {
     printline()
     cat("Model 2:\n")
     print(Model(x$call$model2))
-
 }
 
 ##' @export
@@ -422,7 +418,6 @@ predict.twostage.lvmfit <- function(object,
 ##' @param control1 optimization parameters for model 1
 ##' @param control2 optimization parameters for model 1
 ##' @param knots.boundary boundary points for natural cubic spline basis
-##' @param mc.cores number of cores to use for parallel computations
 ##' @param nmix number of mixture components
 ##' @param df spline degrees of freedom
 ##' @param fix automatically fix parameters for identification (TRUE)
@@ -432,19 +427,22 @@ predict.twostage.lvmfit <- function(object,
 ##' @param messages print information (>0)
 ##' @param ... additional arguments to lower level functions
 ##' @examples
-##' \donttest{ ## Reduce Ex.Timings
-##' m1 <- lvm( x1+x2+x3 ~ u1, latent= ~u1)
+##' \donttest{ ## Reduce Ex.Timings##'
+##' m1 <- lvm( x1+x2+x3 ~ u, latent= ~u)
 ##' m2 <- lvm( y ~ 1 )
 ##' m <- functional(merge(m1,m2), y ~ u, value=function(x) sin(x)+x)
 ##' distribution(m, ~u1) <- uniform.lvm(-6,6)
 ##' d <- sim(m,n=500,seed=1)
 ##' nonlinear(m2) <- y~u1
-##' val <- twostageCV(m1, m2, data=d, std.err=FALSE, df=2:6, nmix=1:2,
-##'                   nfolds=2, mc.cores=1)
-##' val
+##' if (requireNamespace('mets', quietly=TRUE)) {
+##'   set.seed(1)
+##'   val <- twostageCV(m1, m2, data=d, std.err=FALSE, df=2:6, nmix=1:2,
+##'                   nfolds=2)
+##'   val
+##' }
 ##' }
 twostageCV <- function(model1, model2, data, control1=list(trace=0), control2=list(trace=0),
-                knots.boundary, mc.cores=1, nmix=1:4, df=1:9, fix=TRUE, std.err=TRUE,
+                knots.boundary, nmix=1:4, df=1:9, fix=TRUE, std.err=TRUE,
                 nfolds=5, rep=1, messages=0, ...) {
 
     F <- nonlinear(model2)
@@ -452,7 +450,7 @@ twostageCV <- function(model1, model2, data, control1=list(trace=0), control2=li
         stop("Specify at least one nonlinear association in 'model2' (using the 'nonlinear' method)")
     }
     form <- as.formula(paste(names(F)[1], "~", x=F[[1]]$x))
-    
+
     op <- options(warn=-1)
     if (fix) {
         model1 <- baptize(fixsome(model1, param="relative"))
@@ -477,19 +475,11 @@ twostageCV <- function(model1, model2, data, control1=list(trace=0), control2=li
     ee <- list(e1a)
 
     nmix <- setdiff(nmix,1)
-    if (mc.cores>1) {
-        val <- parallel::mclapply(as.list(nmix), function(k) {
-            if (messages>0) cat("Fitting mixture model with", k, "components\n")
-            mixture(model1, k=k, data=data, control=c(control1,list(start=startf(k))))
-        }, mc.cores=mc.cores)
-        ee <- c(ee, val)
-    } else {
-        for (k in nmix) {
-            if (messages>0) cat("Fitting mixture model with", k, "components\n")
-            ee <- c(ee, list(mixture(model1, k=k, data=data,
-                                     control=c(control1,list(start=startf(k))))))
-        }
-    }
+    val <- future_lapply(as.list(nmix), function(k) {
+      if (messages>0) cat("Fitting mixture model with", k, "components\n")
+      mixture(model1, k=k, data=data, control=c(control1,list(start=startf(k))))
+    }, ...)
+    ee <- c(ee, val)
     AIC1 <- unlist(lapply(ee,AIC))
     names(AIC1) <- c(1,nmix)
     ii <- which.min(AIC1)
@@ -510,19 +500,18 @@ twostageCV <- function(model1, model2, data, control1=list(trace=0), control2=li
     } else {
         f0 <- function(data) list(e0=mixture(model1,data=data,k=e1$k,control=c(control1,list(start=coef(e1)))))
     }
-    
+
     ff <- lapply(MM,
                  function(m) function(data,e0,...)  twostage(e0,m,data=data,std.derr=FALSE))
-    a <- cv(ff,data=data,K=nfolds,rep=rep,mc.cores=mc.cores,shared=f0)
-    M <- MM[[which.min(coef(a))]]
-    e2 <- twostage(e1,M,data,control=control2, std.err=std.err)
+    a <- cv(ff, data=data, K=nfolds, rep=rep, shared=f0)
+    M <- MM[[which.min(a$coef)]]
+    e2 <- twostage(e1, M, data, control=control2, std.err=std.err)
     options(op)
-    res <- list(AIC1=cbind(AIC1), model1=e1, model2=e2, cv=coef(a), knots=c(list(NA),Knots),
+    res <- list(AIC1=cbind(AIC1), model1=e1, model2=e2, cv=a$coef, knots=c(list(NA), Knots),
                 nfolds=nfolds, rep=rep)
     structure(res, class="twostageCV")
 }
 
-
 ##' @export
 print.twostageCV <- function(x,...) {
     printline(70)
@@ -535,6 +524,7 @@ print.twostageCV <- function(x,...) {
     splinedf <- unlist(lapply(x$knots,function(x) if (any(is.na(x))) return(1) else length(x)-1))
     cat("Selected spline model degrees of freedom: ", splinedf[i2] ,"\n", sep="")
     knots <- rbind(x$knots[[i2]])
+    if (is.na(knots[1])) knots <- "none"
     cat("Knots:", paste(formatC(knots,...) , collapse=" "), "\n\n")
     rmse <- x$cv
     rownames(rmse) <- paste0("df:",splinedf)
@@ -542,7 +532,11 @@ print.twostageCV <- function(x,...) {
     print(rmse)
     printline(70)
     cat("\n")
-    print(CoefMat(x$model2,...),quote=FALSE)
+    if (!inherits(x$model2, "lvmfit")) {
+        print(x$model2$coefmat, quote=FALSE)
+    } else {
+        print(CoefMat(x$model2,...),quote=FALSE)
+    }
 }
 
 ##' @export
@@ -550,24 +544,32 @@ coef.twostageCV <- function(object,...) {
     coef(Model(object),...)
 }
 
+##' @export
 vcov.twostageCV <- function(object,...) {
     vcov(Model(object),...)
 }
 
-iid.twostageCV <- function(object,...) {
-    iid(Model(object),...)
+##' @export
+IC.twostageCV <- function(x,...) {
+    IC(Model(x),...)
 }
 
+##' @export
 Model.twostageCV <- function(x,...) {
     x$model2
 }
 
-
 ##' @export
 summary.twostageCV <- function(object,...) {
-    with(object, list(model1=summary(model1),
-                      AIC1=AIC1, cv=cv, knots=knots,
-                      model2=summary(model2)))
+    res <- with(object, list(model1=summary(model1),
+                                AIC1=AIC1, cv=cv, knots=knots,
+                                model2=summary(model2)))
+    structure(res, class="summary.twostageCV")
+}
+
+##' @export
+print.summary.twostageCV <- function(x,...) {
+    print.twostageCV(x, ...)
 }
 
 ##' @export
